#include <Ucglib.h>
    
// Create display and define the pins:
Ucglib_ST7735_18x128x160_HWSPI ucg(8, 10, 9);  // (A0=8, CS=10, RESET=9)
// The rest of the pins are pre-selected as the default hardware SPI for Arduino Uno (SCK=13 and SDA=11)

#define LED 2 // Ledul se afla la pinul 3
#define BUZZER 3 // Buzzer-ul se afla la pinul 3

#define UP A0
#define DOWN A1
#define LEFT A2
#define RIGHT A3
#define SELECT A4
#define INCREMENT A5
#define DELETE 7

#define MAX_NUMBER 4
#define CELL_HEIGHT 32
#define CELL_WIDTH 40

int number; //numarul care se scrie
int cell_x, cell_y; // coordonatele celulei in care se scrie
int number_pos_x, number_pos_y; //coordonatele numarului care se scrie
int sudoku_matrix[MAX_NUMBER][MAX_NUMBER];

void resetGame(void) {

 ucg.clearScreen();  // Clear the screen

  // Set display orientation:
  ucg.setRotate90();  // Put 90, 180 or 270, or comment to leave default
  ucg.setFont(ucg_font_inr19_mr);

// GRID //

  for(int i = 0 ; i < 4 ; i++) {
    for(int j = 0 ; j < 4 ; j++) {
       sudoku_matrix[i][j] = 0;
    } 
  } 

  for(int i = 0 ; i < 4 ; i++) {
    // Draw horizontal lines
    ucg.setColor(255, 255, 255);  // Set color (0,R,G,B)
    ucg.drawHLine(0, (32*i), 160);  // Start from left pixel (x,y,lenght)  
  } 
  
  // Draw horizontal line:
  ucg.setColor(255, 255, 255);  // Set color (0,R,G,B)
  ucg.drawHLine(0, 127, 160);  // Start from left pixel (x,y,lenght)

  for(int i = 0 ; i < 4 ; i++) {
    // Draw vertical lines
    ucg.setColor(255, 255, 255);  // Set color (0,R,G,B)
    ucg.drawVLine((40*i), 0, 128);  // Start from left pixel (x,y,lenght)
  } 
  
  // Draw vertical line:
  ucg.setColor(255, 255, 255);  // Set color (0,R,G,B)
  ucg.drawVLine(159, 0, 128);  // Start from left pixel (x,y,lenght)

  // Print initial numbers:
  ucg.setColor(0, 255, 0, 0);  // Set color (0,R,G,B)
  ucg.setColor(1, 0, 0, 0);  // Set color of text background (1,R,G,B)
  ucg.setPrintPos(12,25);  // Set position (x,y)
  ucg.print("3");  // Print text or value
  sudoku_matrix[0][0] = 3;

  ucg.setColor(0, 255, 0, 0);  // Set color (0,R,G,B)
  ucg.setColor(1, 0, 0, 0);  // Set color of text background (1,R,G,B)
  ucg.setPrintPos(52,89);  // Set position (x,y)
  ucg.print("1");  // Print text or value
  sudoku_matrix[2][1] = 1;

  ucg.setColor(0, 255, 0, 0);  // Set color (0,R,G,B)
  ucg.setColor(1, 0, 0, 0);  // Set color of text background (1,R,G,B)
  ucg.setPrintPos(92,57);  // Set position (x,y)
  ucg.print("2");  // Print text or value
  sudoku_matrix[1][2] = 2;

  ucg.setColor(0, 255, 0, 0);  // Set color (0,R,G,B)
  ucg.setColor(1, 0, 0, 0);  // Set color of text background (1,R,G,B)
  ucg.setPrintPos(132,121);  // Set position (x,y)
  ucg.print("2");  // Print text or value
  sudoku_matrix[3][3] = 2;  

  
  // Valori initiale
  number = 0;
  number_pos_x = 12;
  number_pos_y = 25;
  cell_x = 0;
  cell_y = 0;

  ucg.setColor(0, 0, 255);
  ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);
}
void setup(void)  // Start of setup
{
  pinMode(LED,OUTPUT);
  pinMode(BUZZER, OUTPUT);

  pinMode(UP, INPUT_PULLUP);
  pinMode(DOWN, INPUT_PULLUP);
  pinMode(LEFT, INPUT_PULLUP);
  pinMode(RIGHT, INPUT_PULLUP);
  pinMode(SELECT, INPUT_PULLUP);
  pinMode(INCREMENT, INPUT_PULLUP);
  pinMode(DELETE, INPUT_PULLUP);

  // Display setup:
  
  // Select a type of text background:
  //ucg.begin(UCG_FONT_MODE_TRANSPARENT);  // It doesn't overwrite background, so it's a mess for text that changes
  ucg.begin(UCG_FONT_MODE_SOLID);  // It writes a background for the text. This is the recommended option

  resetGame();
}  // End of setup

void resetNumber(void)
{
  number = 0;
}

void resetPosition(void)
{
  number_pos_x = 12;
  number_pos_y = 25;
  cell_x = 0;
  cell_y = 0;
}

bool canWrite(int x, int y)
{
  int row, col;
  row = y / CELL_HEIGHT;
  col = x / CELL_WIDTH;

  if (sudoku_matrix[row][col] != 0) {
    return false;
  }
  
  return true;
}

bool goodNumber(int number, int pos_x, int pos_y)
{
  int col, row, square;
  row = pos_y / CELL_HEIGHT;
  col = pos_x / CELL_WIDTH;

  /* Se determina careul din care face parte elementul */
  if (row >= 0 && row <= 1) {
    if (col >= 0 && col <= 1) {
      square = 1;
    } else if (col >= 2 && col <= 3) {
      square = 2;
    }
  } else if (row >= 2 && row <= 3) {
    if (col >= 0 && col <= 1) {
      square = 3;
    } else  if (col >= 2 && col <= 3) {
      square = 4;
    }
  }

  /* Se verifica ca pe acelasi rand/aceeasi coloana sa nu mai existe un numar identic */
  for (int i = 0 ; i < MAX_NUMBER; i++) {
    if ((i != col) && (sudoku_matrix[row][i] == number)){
     return false;  
    }

     if ((i != row) && (sudoku_matrix[i][col] == number)){
      return false;  
    }
  }

  /* Se verifica ca in acelasi careu sa nu mai existe un numar identic */
  for (int i = 0 ; i < 2 ; i++) {
    int cur_line = i;
    if (square > 2) {
      cur_line += 2;
    }
    for (int j = 0 ; j < 2 ; j++) {
      int cur_col = j;
      if (square % 2 == 0) {
        cur_col += 2;
      }
      if (cur_line != row && cur_col != col && sudoku_matrix[cur_line][cur_col] == number) {
        return false;  
      }
    }
  }

  sudoku_matrix[row][col] = number;
  return true;
}

void deleteNumber(int x, int y)
{
  int row, col;
  row = y / CELL_HEIGHT;
  col = x / CELL_WIDTH;

  if ((row == 0 && col == 0) ||
      (row == 1 && col == 2) ||
      (row == 2 && col == 1) ||
      (row == 3 && col == 3)) {
    return;      
  }
  sudoku_matrix[row][col] = 0;
  ucg.setColor(0, 255, 255, 255); 
  ucg.setColor(1, 0, 0, 0);
  ucg.setPrintPos(x,y);
  ucg.print(" ");
}

bool endGame(void) {

  int i, j;
  for (i = 0 ; i < MAX_NUMBER; i++) {
    for (j = 0 ; j < MAX_NUMBER; j++) {
      if (sudoku_matrix[i][j] == 0) {
        return false;  
      }  
    }  
  }

  return true;
}
void loop(void)  // Start of loop
{
  if (digitalRead(UP) == LOW){
    resetNumber();
    ucg.setColor(255, 255, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);

    if ((number_pos_y - CELL_HEIGHT) > 0) {
      number_pos_y -= CELL_HEIGHT;
      cell_y -= CELL_HEIGHT;
    }
    ucg.setColor(0, 0, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);
  }

  if (digitalRead(DOWN) == LOW){
    resetNumber();
    ucg.setColor(255, 255, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);

    if ((number_pos_y + CELL_HEIGHT) < 127) {
      number_pos_y += CELL_HEIGHT;
      cell_y += CELL_HEIGHT;
    }
    ucg.setColor(0, 0, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);
  }

  if (digitalRead(LEFT) == LOW){
    resetNumber();
    ucg.setColor(255, 255, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);

    if ((number_pos_x - CELL_WIDTH) > 0) {
      number_pos_x -= CELL_WIDTH;
      cell_x -= CELL_WIDTH;
    }
    ucg.setColor(0, 0, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);
  }
  
  if (digitalRead(RIGHT) == LOW){
    resetNumber();
    ucg.setColor(255, 255, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);

    if ((number_pos_x + CELL_WIDTH) < 159) {
      number_pos_x += CELL_WIDTH;
      cell_x += CELL_WIDTH;
    }
    ucg.setColor(0, 0, 255);
    ucg.drawFrame(cell_x, cell_y, CELL_WIDTH, CELL_HEIGHT);
  }
  
  if (digitalRead(INCREMENT) == LOW && canWrite(number_pos_x, number_pos_y)){
    
    if (number < MAX_NUMBER) {
      number++;
    }
    ucg.setColor(0, 255, 255, 255); 
    ucg.setColor(1, 0, 0, 0);
    ucg.setPrintPos(number_pos_x,number_pos_y);
    ucg.print(number);
    delay(300);
    ucg.setColor(0, 255, 255, 255); 
    ucg.setColor(1, 0, 0, 0);
    ucg.setPrintPos(number_pos_x,number_pos_y);
    ucg.print(" ");
  }

  if (digitalRead(SELECT) == LOW && number != 0){
    if (canWrite(number_pos_x, number_pos_y) && goodNumber(number, number_pos_x, number_pos_y)) {
      ucg.setColor(0, 255, 255, 255); 
      ucg.setColor(1, 0, 0, 0);
      ucg.setPrintPos(number_pos_x,number_pos_y);
      ucg.print(number);
    } else {
       digitalWrite(LED, HIGH);
       tone(BUZZER, 600);
       delay(200);
       digitalWrite(LED, LOW);
       noTone(BUZZER);
    }
    resetNumber();
  }

  if (digitalRead(DELETE) == LOW){
    resetNumber();
    deleteNumber(number_pos_x, number_pos_y);
  }

  delay(200);

  if (endGame()) {
    ucg.clearScreen();
    ucg.setRotate90();
    ucg.setColor(0, 255, 255, 255); 
    ucg.setColor(1, 0, 0, 0);
    ucg.setPrintPos(7,60);
    ucg.print("FELICITARI!"); 
    delay(350);
    resetGame();
  }
  
}  // End of loop
